"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.EnergyTransferRequest = exports.State = exports.UpdateStatusCode = exports.TransferValidationStatus = void 0;
const ethers_1 = require("ethers");
var TransferValidationStatus;
(function (TransferValidationStatus) {
    TransferValidationStatus["Pending"] = "pending";
    TransferValidationStatus["Valid"] = "valid";
    TransferValidationStatus["Invalid"] = "invalid";
    TransferValidationStatus["Error"] = "error";
})(TransferValidationStatus = exports.TransferValidationStatus || (exports.TransferValidationStatus = {}));
var UpdateStatusCode;
(function (UpdateStatusCode) {
    UpdateStatusCode["Success"] = "success";
    UpdateStatusCode["NoValidator"] = "noValidator";
    UpdateStatusCode["NotPending"] = "notPending";
})(UpdateStatusCode = exports.UpdateStatusCode || (exports.UpdateStatusCode = {}));
var State;
(function (State) {
    State["Skipped"] = "Skipped";
    State["IssuanceAwaiting"] = "IssuanceAwaiting";
    State["IssuanceInProgress"] = "IssuanceInProgress";
    State["Issued"] = "Issued";
    State["PersistenceAwaiting"] = "PersistenceAwaiting";
    State["Persisted"] = "Persisted";
    State["ValidationAwaiting"] = "ValidationAwaiting";
    State["ValidationInProgress"] = "ValidationInProgress";
    State["Validated"] = "Validated";
    State["TransferAwaiting"] = "TransferAwaiting";
    State["TransferInProgress"] = "TransferInProgress";
    State["Transferred"] = "Transferred";
    State["IssuanceError"] = "IssuanceError";
    State["TransferError"] = "TransferError";
    State["ValidationError"] = "ValidationError";
    State["ValidationInvalid"] = "ValidationInvalid";
})(State = exports.State || (exports.State = {}));
class EnergyTransferRequest {
    constructor(attrs) {
        this.attrs = attrs;
    }
    get certificateId() {
        return this.attrs.certificateId;
    }
    get id() {
        return this.attrs.id;
    }
    get sites() {
        return {
            buyerAddress: this.attrs.buyerAddress,
            sellerAddress: this.attrs.sellerAddress
        };
    }
    get volume() {
        return this.attrs.volume;
    }
    issuanceStarted() {
        this.nextState(State.IssuanceAwaiting, State.IssuanceInProgress);
    }
    issuanceError(error) {
        this.nextState(State.IssuanceInProgress, State.IssuanceError);
        this.attrs.processError = error;
    }
    issuanceFinished(certificateId) {
        this.nextState(State.IssuanceInProgress, State.Issued);
        this.attrs.certificateId = certificateId;
        this.nextState(State.Issued, State.PersistenceAwaiting);
    }
    persisted() {
        this.nextState(State.PersistenceAwaiting, State.Persisted);
        this.nextState(State.Persisted, State.ValidationAwaiting);
    }
    startValidation(validatorNames) {
        this.nextState(State.ValidationAwaiting, State.ValidationInProgress);
        if (validatorNames.length === 0) {
            this.nextState(State.ValidationInProgress, State.Validated);
            this.nextState(State.Validated, State.TransferAwaiting);
            return;
        }
        this.attrs.validationStatusRecord = validatorNames.reduce((status, command) => ({
            ...status,
            [command]: TransferValidationStatus.Pending
        }), {});
    }
    validationError(error) {
        this.nextState(State.ValidationInProgress, State.ValidationError);
        this.attrs.processError = error;
    }
    updateValidationStatus(validatorName, status) {
        const currentStatus = this.attrs.validationStatusRecord[validatorName];
        if (!currentStatus) {
            return UpdateStatusCode.NoValidator;
        }
        if (currentStatus !== TransferValidationStatus.Pending) {
            return UpdateStatusCode.NotPending;
        }
        this.attrs.validationStatusRecord[validatorName] = status;
        if (this.attrs.state === State.ValidationInProgress) {
            const statuses = Object.values(this.attrs.validationStatusRecord);
            if (statuses.some((s) => s === TransferValidationStatus.Error)) {
                this.nextState(State.ValidationInProgress, State.ValidationError);
            }
            else if (statuses.some((s) => s === TransferValidationStatus.Invalid)) {
                this.nextState(State.ValidationInProgress, State.ValidationInvalid);
            }
            else if (statuses.every((s) => s === TransferValidationStatus.Valid)) {
                this.nextState(State.ValidationInProgress, State.Validated);
                this.nextState(State.Validated, State.TransferAwaiting);
            }
        }
        return UpdateStatusCode.Success;
    }
    transferStarted() {
        this.nextState(State.TransferAwaiting, State.TransferInProgress);
    }
    transferError(error) {
        this.nextState(State.TransferInProgress, State.TransferError);
        this.attrs.processError = error;
    }
    transferFinished() {
        this.nextState(State.TransferInProgress, State.Transferred);
    }
    toAttrs() {
        return this.attrs;
    }
    toPublicAttrs() {
        return {
            buyerAddress: this.attrs.buyerAddress,
            sellerAddress: this.attrs.sellerAddress,
            id: this.attrs.id,
            transferDate: this.attrs.transferDate,
            volume: this.attrs.volume,
            certificateData: this.attrs.certificateData
        };
    }
    static fromAttrs(attrs) {
        return new EnergyTransferRequest(attrs);
    }
    static newAttributes(params) {
        return {
            ...params,
            createdAt: new Date(),
            updatedAt: new Date(),
            certificateId: null,
            validationStatusRecord: {},
            state: ethers_1.BigNumber.from(params.volume).eq(0) ? State.Skipped : State.IssuanceAwaiting,
            processError: null
        };
    }
    nextState(expectedCurrentState, nextState) {
        if (this.attrs.state !== expectedCurrentState) {
            throw new Error(`ETR ${this.attrs.id} is in invalid state to execute method. Expected ${expectedCurrentState}, current: ${this.attrs.state}`);
        }
        this.attrs.state = nextState;
    }
}
exports.EnergyTransferRequest = EnergyTransferRequest;
//# sourceMappingURL=EnergyTransferRequest.js.map